//==============================================================================
//
// Title:		LabJack UART
// Purpose:		A short description of the application.
//
// Created on:	3/3/2017 at 3:25:04 PM by Shakeel.
// Copyright:	. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "LabJack UART.h"
#include "toolbox.h"
#include "C:\Program Files (x86)\LabJack\Drivers\LabJackUD.h"

//==============================================================================
// Constants

//==============================================================================
// Types

//==============================================================================
// Static global variables

static int panelHandle = 0;

//==============================================================================
// Static functions

long glngHandleptr;
//==============================================================================
// Global variables
char err[255];
char ch;
char ErrorMsg[1024];

//==============================================================================
// Global functions

int ErrorHandler (LJ_ERROR lngErrorcode, long lngLineNumber, long lngIteration)
{
	char err[255];

	if (lngErrorcode != LJE_NOERROR)
	{
		ErrorToString(lngErrorcode,err);
		printf("Error number = %d\n",lngErrorcode);
		printf("Error string = %s\n",err);
		printf("Source line number = %d\n",lngLineNumber);
		printf("Iteration = %d\n\n",lngIteration);
		if(lngErrorcode > LJE_MIN_GROUP_ERROR)
		{
			return 0;
		}
		return 0;
   }
   return 1;
}

int FindAddress(long SerialNumber,int findOtherNotAddress)
{
	//long dAddress=0.0;
	long pNumFound;
	long pSerialNumbers[128];
	long pIDs[128];
	double pAddresses[128];
	ListAll(LJ_dtU3,LJ_ctUSB, &pNumFound,pSerialNumbers,pIDs, pAddresses); 
	for(int i=0;i<pNumFound;++i)
	{
		if(pSerialNumbers[i]==SerialNumber)
		{
			if(!findOtherNotAddress)
			{
				//dAddress=pAddresses[i];
				return 1;
			}
		}
		else
		{
			if(findOtherNotAddress)
			{
				//dAddress=pAddresses[i];
				return 1;
			}
		}
	}
	
	return 0;
}

int OpenLabJackComms_Ex(long *exlngHandle)
{
	long lngErrorcode; 
	//Open the first found LabJack UE9.
	lngErrorcode = OpenLabJack (LJ_dtU3, LJ_ctUSB, "", 1, exlngHandle);
	//lngErrorcode = OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, "192.168.1.209", 0, &lngHandle);
	if(lngErrorcode!=LJE_NOERROR)
	{
		ErrorToString(lngErrorcode,err);
	    sprintf(ErrorMsg,"\nError message = %s\n",err);	
		MessagePopup("",ErrorMsg);
		return 0;
	}
	else 
	{
		return 1;
	}
}

int OpenLabJackCommsExViaSN(long *exlngHandleptr,char SerialNumber[30])
{
	long lngErrorcode; 
	long lAddress=atol(SerialNumber);
	
	if(!FindAddress(lAddress,0))
	{
		MessagePopup("","Could Not Find Requested Labjack.");
		return 0;	
	}
	//Open the first found LabJack UE9.
	lngErrorcode = OpenLabJack (LJ_dtU3, LJ_ctUSB, SerialNumber, 0, exlngHandleptr);
	//lngErrorcode = OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, "192.168.1.209", 0, &lngHandle);
	if(lngErrorcode!=LJE_NOERROR)
	{
		ErrorToString(lngErrorcode,err);
	    sprintf(ErrorMsg,"\nError message = %s\n",err);	
		MessagePopup("",ErrorMsg);
		return 0;
	}
	else 
	{
		return 1;
	}
}

void CloseLabJack(void)
{
	Close();	
}


int SetupUartBaudRateandEnableAsynchUART(long *exlngHandleptr,int BaudRate)
{
	long lngErrorcode; 
	double dBaudRate= ((65535) - (48000000/(2 * BaudRate))); 
	unsigned short int usiBaudRate=dBaudRate;
	//double numI2CBytesToWrite;
	double numBytes;
	unsigned char array[256] = {0};
	long parray = (long)&array[0];


	if(OpenLabJackComms_Ex(exlngHandleptr))
	{
		//Start by using the pin_configuration_reset IOType so that all
		//pin assignments are in the factory default condition.
		lngErrorcode = ePut (*exlngHandleptr, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}


		// 1 MHz timer clock base.
		lngErrorcode = ePut(*exlngHandleptr, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc1MHZ_DIV, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}

		// Set clock divisor to 1, so timer clock is 1 MHz.
		lngErrorcode = ePut(*exlngHandleptr, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 1, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}

		// Set timer/counter pin offset to 4. TX and RX appear after any timers and counters on U3
		// hardware rev 1.30.  We have no timers or counters enabled, so TX=FIO4 and RX=FIO5.
		lngErrorcode = ePut(*exlngHandleptr, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 4, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}

		// Set data rate for 9600 bps communication.
		lngErrorcode = ePut(*exlngHandleptr, LJ_ioPUT_CONFIG, LJ_chASYNCH_BAUDFACTOR, dBaudRate, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}

		// Enable UART.
		lngErrorcode = ePut(*exlngHandleptr, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_ENABLE, 1, 0);
		if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
		{
			return 0;		
		}
	}
	else 
	{
		return 0;	
	}
	return 1;
		
}

int WriteUART(long exlngHandleptr,unsigned char *UART_Tx)
{
	long lngErrorcode; 
	double NumberofBytes=strlen(UART_Tx);
	long parray = (long)&UART_Tx[0];
	
	//Write data.
	lngErrorcode=eGet(exlngHandleptr, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_TX, &NumberofBytes, parray);	
	if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
	{
		return 0;		
	}
	return 1;
}

int ReadUART(long exlngHandleptr,char *UART_Rx)
{
	long lngErrorcode; 
	
	long parray = (long)&UART_Rx[0]; 
	double NumberofBytes=1024;
	//Write data.   eGet
	lngErrorcode=eGet(exlngHandleptr, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_RX, &NumberofBytes, parray);
	if(ErrorHandler(lngErrorcode, __LINE__, 0)==0)
	{
		return 0;		
	}
	return 1;
}



void WriteUARTTest(long exlngHandleptr)
{
	long lngErrorcode;
	double numBytes;
	unsigned char array[256] = {0};
	long parray = (long)&array[0];
	numBytes = 2;
	array[0] = 12;
	array[1] = 34;
	lngErrorcode = eGet(exlngHandleptr, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_TX, &numBytes, parray);
	ErrorHandler(lngErrorcode, __LINE__, 0);
}

void ReadUARTTest(long exlngHandleptr)
{
	long lngErrorcode;
	double numBytes;
	unsigned char array[256] = {0};
	long parray = (long)&array[0];
	
	// Read 2 bytes.
	numBytes = 9999;  //Dummy values so we can see them change.
	array[0] = 222;
	array[1] = 222;
	lngErrorcode = eGet(exlngHandleptr, LJ_ioASYNCH_COMMUNICATION, LJ_chASYNCH_RX, &numBytes, parray);
	ErrorHandler(lngErrorcode, __LINE__, 0);
}




/// HIFN The main entry-point function.
int main (int argc, char *argv[])
{
	int error = 0;
	
	/* initialize and load resources */
	nullChk (InitCVIRTE (0, argv, 0));
	errChk (panelHandle = LoadPanel (0, "LabJack UART.uir", PANEL));
	
	/* display the panel and run the user interface */
	errChk (DisplayPanel (panelHandle));
	errChk (RunUserInterface ());

Error:
	/* clean up */
	if (panelHandle > 0)
		DiscardPanel (panelHandle);
	return 0;
}

//==============================================================================
// UI callback function prototypes

/// HIFN Exit when the user dismisses the panel.
int CVICALLBACK panelCB (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	if (event == EVENT_CLOSE)
		QuitUserInterface (0);
	return 0;
}

int CVICALLBACK WriteToUART (int panel, int control, int event,
							 void *callbackData, int eventData1, int eventData2)
{
	char UART_Tx[1024];
	switch (event)
	{
		case EVENT_COMMIT:
				GetCtrlVal (panelHandle, PANEL_TEXTBOX, UART_Tx);
				WriteUART(glngHandleptr,UART_Tx);
				//WriteUARTTest(glngHandleptr); 
			break;
	}
	return 0;
}

int CVICALLBACK ReadToUART (int panel, int control, int event,
							void *callbackData, int eventData1, int eventData2)
{
	char UART_Rx[1024]={0};
	switch (event)
	{
		case EVENT_COMMIT:
				ReadUART(glngHandleptr,UART_Rx);
				///ReadUARTTest(glngHandleptr); 
				ResetTextBox(panelHandle,PANEL_READDATA,UART_Rx);
			break;
	}
	return 0;
}

int CVICALLBACK OpenUARTComms (int panel, int control, int event,
							   void *callbackData, int eventData1, int eventData2)
{
	int BaudRate;
	switch (event)
	{
		case EVENT_COMMIT:
				GetCtrlVal (panelHandle, PANEL_BAUDRATE, &BaudRate);
				if(SetupUartBaudRateandEnableAsynchUART(&glngHandleptr,BaudRate))
				{
					SetCtrlVal(panelHandle,PANEL_CONNECTEDLED,1);	
				}
				else
				{
					SetCtrlVal(panelHandle,PANEL_CONNECTEDLED,0);		
				}
				
			break;
	}
	return 0;
}
